#pragma once

#include <WmlVector3.h>
#include <vector>

class ParticleSet
{
public:
	ParticleSet(void);
	~ParticleSet(void);

	virtual unsigned int ParticleCount() const = 0;
	virtual Wml::Vector3f GetParticlePosition(int i) const = 0;
	virtual void SetParticlePosition( int i, const Wml::Vector3f & vPosition) = 0;

	void InitializeParticles();
	void UpdateParticles();

	unsigned int N() const { return (unsigned int)m_X.size(); }

	const Wml::Vector3f & X(int i) const { return m_X[i]; }
	const Wml::Vector3f & V(int i) const { return m_V[i]; }
	const Wml::Vector3f & A(int i) const { return m_A[i]; }
	const Wml::Vector3f & F(int i) const { return m_F[i]; }
	float M(int i) const { return m_M[i]; }

	Wml::Vector3f & X(int i) { return m_X[i]; }
	Wml::Vector3f & V(int i) { return m_V[i]; }
	Wml::Vector3f & A(int i) { return m_A[i]; }
	Wml::Vector3f & F(int i) { return m_F[i]; }
	float & M(int i) { return m_M[i]; }

	void DrawParticles();

protected:
	std::vector<Wml::Vector3f> m_X;		// position
	std::vector<Wml::Vector3f> m_V;		// velocity
	std::vector<Wml::Vector3f> m_A;		// acceleration
	std::vector<Wml::Vector3f> m_F;		// force
	std::vector<float> m_M;				// mass
};



class Simulator
{
public:
	Simulator();
	~Simulator();

	void SetParticles(ParticleSet * pParticles);
	virtual void InitializeSimulator() = 0;

	void StepSimulation(float dt);
	virtual void ComputeForces() = 0;

protected:
	ParticleSet * m_pParticles;
};



class OriginSpringSimulator : public Simulator
{
public:
	OriginSpringSimulator(float fK = 1.0f, float fB = 0.1f);

	virtual void InitializeSimulator();
	virtual void ComputeForces();

protected:
	std::vector<Wml::Vector3f> m_vRestPos;
	float m_fK;		// spring stiffness
	float m_fB;		// damping constant
};